
///////////////////////////////////////////////////////////////////////////////
//
//  ace_udf.ch
//
//  This is the main header file to translate Xbase++ database field access
//  commands and functions to the appropriate ACE wrapper functions.
//
//  Copyright (C) 1998-2001, Boris Borzic
//  All rights reserved.
//
//  Author:        Boris Borzic
//  E-mail:        support@sqlexpress.net
//  Homepage:      http://www.SQLExpress.net
//
//  Last Modified: Mar 28, 2001
//
///////////////////////////////////////////////////////////////////////////////
//
//  ** NOTES **
//
//  (1) This file must be included in all your PRGs! It is used by the
//      pre-processor to translate field access commands to the appropriate
//      ACE wrapper function. To see how the translation works, compile your
//      PRG with the /P parameter and look at the generated .PPO file.
//
//  (2) You will need to modify this file by appending and/or replacing the
//      TABLE1, TABLE2, TABLE3,... alias names with the appropriate literal
//      alias names that you use in your programs. If you don't use literal
//      alias names then even better, you don't need to do anything except
//      delete all references to TABLE1, TABLE2, TABLE3 in this file.
//
//      This is a literal alias expression (you need to add 'CUSTOMER' to
//      ACE_UDF.CH for it to be preprocessed correctly):
//          CUSTOMER->LASTNAME := "Jones"
//
//      This is an extended alias expression (no changes needed to ACE_UDF.CH):
//          cMyAlias := 'CUST1'
//          (cMyAlias)->LASTNAME := "Jones"
//
//  (3) Unfortunately the preprocessor can't translate expressions that are
//      going to be macroed, therefore you will need to change all field
//      references inside macros to FieldGet()/FieldPut() or
//      FieldWGet()/FieldWPut() functions.
//
//  (4) The proprocessor gets mixed up with complex REPLACE commands. You
//      should change these to appropriate field assignments, ex:
//          REPLACE xx->field1 WITH yy->field1
//          xx->field1 := yy->field1
//
//  (5) The preprocessor also gets mixed up with complex commands such as this
//
//      if xx->field1 .and. yy->field2 .and. !xx->(eof())
//
//      The solution is to simply put parentheses around all fields and
//      aliased functions:
//
//      if (xx->field1) .and. (yy->field2) .and. !(xx->(eof()))
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __ACE_UDF__
#define __ACE_UDF__

// since ADS doesn't care about codeblocks we need to redefine
// these functions to remove the blockify macros

#xtranslate CreateIndex(<cIndexFile>,<cIndexKey>[,<bIndexKey>][,<lUnique>]) => ;
   OrdCreate(<cIndexFile>,NIL,<cIndexKey>[,<bIndexKey>][,<lUnique>])

#xtranslate DbSetOrder( <nIndex> ) => OrdSetFocus( <nIndex> )
#xtranslate DbSetIndex( <nIndex> ) => OrdSetFocus( <nIndex> )
#xtranslate DbClearIndex()         => OrdListClear()

#command  INDEX ON <key> [TAG <(cOrdName)> ] TO <(cOrdBagName)> ;
           [   FOR <for>] ;
           [ WHILE <whl>] ;
           [  NEXT <nxt>] ;
           [RECORD <rec>] ;
           [  EVAL <eval>];
           [ EVERY <evry>];
           [ <rst: REST>] ;
           [ <all: ALL> ] ;
           [<uniq: UNIQUE>] ;
           [ <asc: ASCENDING>] ;
           [<desc: DESCENDING>] ;
           [ <add: ADDITIVE>] ;
      =>  ordCondSet( <"for">, <"for">,[<.all.>], ;
                      <"whl">, <"eval">, <evry> , ;
                      RECNO(), <nxt>, <rec>, ;
                      [<.rst.>], [<.desc.>] ) ;;
          ordCreate( <(cOrdBagName)>, <(cOrdName)>, ;
                     <"key">, <"key">, [<.uniq.>], <.add.> )

#command INDEX ON <key> TAG <(cOrdName)> [TO <(cOrdBagName)>] ;
           [   FOR <for>] ;
           [ WHILE <whl>] ;
           [  NEXT <nxt>] ;
           [RECORD <rec>] ;
           [  EVAL <eval>];
           [ EVERY <evry>];
           [ <rst: REST>] ;
           [ <all: ALL> ] ;
           [<uniq: UNIQUE>] ;
           [ <asc: ASCENDING>] ;
           [<desc: DESCENDING>] ;
           [ <add: ADDITIVE>] ;
      =>  ordCondSet( <"for">, <"for">,[<.all.>], ;
                      <"whl">, <"eval">, <evry> , ;
                      RECNO(), <nxt>, <rec>, ;
                      [<.rst.>], [<.desc.>] ) ;;
          ordCreate(<(cOrdBagName)>, <(cOrdName)>, ;
                    <"key">, <"key">, [<.uniq.>], <.add.> )

#command  INDEX ON <key> TO <(file)> [<uniq: UNIQUE>] ;
      =>  ordCreate(<(file)>, nil, <"key">, <"key">, [<.uniq.>], .f.)

#command  LOCATE ;
          [   FOR <for>] ;
          [ WHILE <whl>] ;
          [  NEXT <nxt>] ;
          [RECORD <rcd>] ;
          [ <rst: REST>] ;
          [ ALL ] ;
          =>  dbLocate( <"for">, <{whl}>, <nxt>, <rcd>, <.rst.> )

#command  APPEND [FROM <(file)>] ;
          [ FIELDS <flds,...>] ;
          [ FOR <for>] ;
          [ ALL ] ;
          => dbImport( <(file)>, { <(flds)> }, <"for"> )

#command  SET DELETED <x:ON,OFF,&>   =>  setDeleted( !Upper(<(x)>)=="ON" )
#command  SET DELETED (<x>)          =>  setDeleted( <x> )



// field access functions
//
#xtranslate  FIELD-><alias>-><f>          =>  FieldWGet(<"alias">,<"f">)
#xtranslate  FIELD-><alias>-><f> ==  <x>  =>  FieldWGet(<"alias">,<"f">) ==  <x>
#xtranslate  FIELD-><alias>-><f> =   <x>  =>  FieldWGet(<"alias">,<"f">) =   <x>
#xtranslate  FIELD-><alias>-><f> #   <x>  =>  FieldWGet(<"alias">,<"f">) !=  <x>
#xtranslate  FIELD-><alias>-><f> !=  <x>  =>  FieldWGet(<"alias">,<"f">) !=  <x>
#xtranslate  FIELD-><alias>-><f> <>  <x>  =>  FieldWGet(<"alias">,<"f">) !=  <x>
#xtranslate  FIELD-><alias>-><f> \<= <x>  =>  FieldWGet(<"alias">,<"f">) \<= <x>
#xtranslate  FIELD-><alias>-><f> >=  <x>  =>  FieldWGet(<"alias">,<"f">) >=  <x>
#xtranslate  FIELD-><alias>-><f> \<  <x>  =>  FieldWGet(<"alias">,<"f">) \<  <x>
#xtranslate  FIELD-><alias>-><f> >   <x>  =>  FieldWGet(<"alias">,<"f">) >   <x>
#xtranslate  FIELD-><alias>-><f> %   <x>  =>  FieldWGet(<"alias">,<"f">) %   <x>
#xtranslate  FIELD-><alias>-><f> **  <x>  =>  FieldWGet(<"alias">,<"f">) **  <x>
#xtranslate  FIELD-><alias>-><f> *   <x>  =>  FieldWGet(<"alias">,<"f">) *   <x>
#xtranslate  FIELD-><alias>-><f> +   <x>  =>  FieldWGet(<"alias">,<"f">) +   <x>
#xtranslate  FIELD-><alias>-><f> -   <x>  =>  FieldWGet(<"alias">,<"f">) -   <x>
#xtranslate  FIELD-><alias>-><f> /   <x>  =>  FieldWGet(<"alias">,<"f">) /   <x>
#xtranslate  FIELD-><alias>-><f> $   <x>  =>  FieldWGet(<"alias">,<"f">) $   <x>
#xtranslate  FIELD-><alias>-><f>.and.<x>  =>  FieldWGet(<"alias">,<"f">).and.<x>
#xtranslate  FIELD-><alias>-><f>.or. <x>  =>  FieldWGet(<"alias">,<"f">).or. <x>
#xtranslate  FIELD-><alias>-><f> :=  <x>  =>  FieldWPut(<"alias">,<"f">,<x>)

#xtranslate  FIELD-><f>          =>  FieldGet(FieldPos(<"f">))
#xtranslate  FIELD-><f> ==  <x>  =>  FieldGet(FieldPos(<"f">)) ==  <x>
#xtranslate  FIELD-><f> =   <x>  =>  FieldGet(FieldPos(<"f">)) =   <x>
#xtranslate  FIELD-><f> #   <x>  =>  FieldGet(FieldPos(<"f">)) !=  <x>
#xtranslate  FIELD-><f> !=  <x>  =>  FieldGet(FieldPos(<"f">)) !=  <x>
#xtranslate  FIELD-><f> <>  <x>  =>  FieldGet(FieldPos(<"f">)) !=  <x>
#xtranslate  FIELD-><f> \<= <x>  =>  FieldGet(FieldPos(<"f">)) \<= <x>
#xtranslate  FIELD-><f> >=  <x>  =>  FieldGet(FieldPos(<"f">)) >=  <x>
#xtranslate  FIELD-><f> \<  <x>  =>  FieldGet(FieldPos(<"f">)) \<  <x>
#xtranslate  FIELD-><f> >   <x>  =>  FieldGet(FieldPos(<"f">)) >   <x>
#xtranslate  FIELD-><f> %   <x>  =>  FieldGet(FieldPos(<"f">)) %   <x>
#xtranslate  FIELD-><f> **  <x>  =>  FieldGet(FieldPos(<"f">)) **  <x>
#xtranslate  FIELD-><f> *   <x>  =>  FieldGet(FieldPos(<"f">)) *   <x>
#xtranslate  FIELD-><f> +   <x>  =>  FieldGet(FieldPos(<"f">)) +   <x>
#xtranslate  FIELD-><f> -   <x>  =>  FieldGet(FieldPos(<"f">)) -   <x>
#xtranslate  FIELD-><f> /   <x>  =>  FieldGet(FieldPos(<"f">)) /   <x>
#xtranslate  FIELD-><f> $   <x>  =>  FieldGet(FieldPos(<"f">)) $   <x>
#xtranslate  FIELD-><f>.and.<x>  =>  FieldGet(FieldPos(<"f">)).and.<x>
#xtranslate  FIELD-><f>.or. <x>  =>  FieldGet(FieldPos(<"f">)).or. <x>
#xtranslate  FIELD-><f> :=  <x>  =>  FieldPut(FieldPos(<"f">),<x>)


// this should handle all alias references enclosed in parentheses...
//
#xtranslate  (<alias>)-><f>          =>  FieldWGet(<alias>,<"f">)
#xtranslate  (<alias>)-><f> ==  <x>  =>  FieldWGet(<alias>,<"f">) ==  <x>
#xtranslate  (<alias>)-><f> =   <x>  =>  FieldWGet(<alias>,<"f">) =   <x>
#xtranslate  (<alias>)-><f> #   <x>  =>  FieldWGet(<alias>,<"f">) !=  <x>
#xtranslate  (<alias>)-><f> !=  <x>  =>  FieldWGet(<alias>,<"f">) !=  <x>
#xtranslate  (<alias>)-><f> <>  <x>  =>  FieldWGet(<alias>,<"f">) !=  <x>
#xtranslate  (<alias>)-><f> \<= <x>  =>  FieldWGet(<alias>,<"f">) \<= <x>
#xtranslate  (<alias>)-><f> >=  <x>  =>  FieldWGet(<alias>,<"f">) >=  <x>
#xtranslate  (<alias>)-><f> \<  <x>  =>  FieldWGet(<alias>,<"f">) \<  <x>
#xtranslate  (<alias>)-><f> >   <x>  =>  FieldWGet(<alias>,<"f">) >   <x>
#xtranslate  (<alias>)-><f> %   <x>  =>  FieldWGet(<alias>,<"f">) %   <x>
#xtranslate  (<alias>)-><f> **  <x>  =>  FieldWGet(<alias>,<"f">) **  <x>
#xtranslate  (<alias>)-><f> *   <x>  =>  FieldWGet(<alias>,<"f">) *   <x>
#xtranslate  (<alias>)-><f> +   <x>  =>  FieldWGet(<alias>,<"f">) +   <x>
#xtranslate  (<alias>)-><f> -   <x>  =>  FieldWGet(<alias>,<"f">) -   <x>
#xtranslate  (<alias>)-><f> /   <x>  =>  FieldWGet(<alias>,<"f">) /   <x>
#xtranslate  (<alias>)-><f> $   <x>  =>  FieldWGet(<alias>,<"f">) $   <x>
#xtranslate  (<alias>)-><f>.and.<x>  =>  FieldWGet(<alias>,<"f">).and.<x>
#xtranslate  (<alias>)-><f>.or. <x>  =>  FieldWGet(<alias>,<"f">).or. <x>
#xtranslate  (<alias>)-><f> :=  <x>  =>  FieldWPut(<alias>,<"f">,<x>)
#xtranslate  (<alias>)->(<x>)        =>  AceWBlock(<alias>,{|| <x>})
#xtranslate  (<alias>)->(RecNo())    =>  AceWRecNo(<alias>)

// put your own literal alias access functions here...
//
#xtranslate  VLASTNICI-><f>          =>  FieldWGet("VLASTNICI",<"f">)
#xtranslate  VLASTNICI-><f> ==  <x>  =>  FieldWGet("VLASTNICI",<"f">) ==  <x>
#xtranslate  VLASTNICI-><f> =   <x>  =>  FieldWGet("VLASTNICI",<"f">) =   <x>
#xtranslate  VLASTNICI-><f> #   <x>  =>  FieldWGet("VLASTNICI",<"f">) !=  <x>
#xtranslate  VLASTNICI-><f> !=  <x>  =>  FieldWGet("VLASTNICI",<"f">) !=  <x>
#xtranslate  VLASTNICI-><f> <>  <x>  =>  FieldWGet("VLASTNICI",<"f">) !=  <x>
#xtranslate  VLASTNICI-><f> \<= <x>  =>  FieldWGet("VLASTNICI",<"f">) \<= <x>
#xtranslate  VLASTNICI-><f> >=  <x>  =>  FieldWGet("VLASTNICI",<"f">) >=  <x>
#xtranslate  VLASTNICI-><f> \<  <x>  =>  FieldWGet("VLASTNICI",<"f">) \<  <x>
#xtranslate  VLASTNICI-><f> >   <x>  =>  FieldWGet("VLASTNICI",<"f">) >   <x>
#xtranslate  VLASTNICI-><f> %   <x>  =>  FieldWGet("VLASTNICI",<"f">) %   <x>
#xtranslate  VLASTNICI-><f> **  <x>  =>  FieldWGet("VLASTNICI",<"f">) **  <x>
#xtranslate  VLASTNICI-><f> *   <x>  =>  FieldWGet("VLASTNICI",<"f">) *   <x>
#xtranslate  VLASTNICI-><f> +   <x>  =>  FieldWGet("VLASTNICI",<"f">) +   <x>
#xtranslate  VLASTNICI-><f> -   <x>  =>  FieldWGet("VLASTNICI",<"f">) -   <x>
#xtranslate  VLASTNICI-><f> /   <x>  =>  FieldWGet("VLASTNICI",<"f">) /   <x>
#xtranslate  VLASTNICI-><f> $   <x>  =>  FieldWGet("VLASTNICI",<"f">) $   <x>
#xtranslate  VLASTNICI-><f>.and.<x>  =>  FieldWGet("VLASTNICI",<"f">).and.<x>
#xtranslate  VLASTNICI-><f>.or. <x>  =>  FieldWGet("VLASTNICI",<"f">).or. <x>
#xtranslate  VLASTNICI-><f> :=  <x>  =>  FieldWPut("VLASTNICI",<"f">,<x>)
#xtranslate  VLASTNICI->(<x>)        =>  AceWBlock("VLASTNICI",{|| <x>})
#xtranslate  VLASTNICI->(RecNo())    =>  AceWRecNo("VLASTNICI")

#endif  // __ACE_UDF__
